local mouseJoint
local boxes = {}

local function addRect(world, x, y, width, height, bodyType, color)
  local box = {}

  box.body = love.physics.newBody(world, x, y, bodyType)
  box.shape = love.physics.newRectangleShape(width, height)
  box.fixture = love.physics.newFixture(box.body, box.shape)
  box.color = color

  box.body:setUserData(box)

  return box
end

function populateWorld(world)
  local w,h = love.graphics.getDimensions()

  for i=1, 20 do
    local x = love.math.random(w * 0.1, w * 0.9)
    local y = love.math.random(h * 0.1, h * 0.9)
    table.insert(boxes, addRect(world, x, y, 80, 80, 'dynamic', variables.boxColor))
  end

  -- floor
  addRect(world, w / 2, h, w, 70, 'static', variables.wallColor)

  -- left wall
  addRect(world, 0, h / 2, 70, h, 'static', variables.wallColor)

  -- right wall
  addRect(world, w, h / 2, 70, h, 'static', variables.wallColor)

  -- ceiling
  addRect(world, w / 2, 0, w, 70, 'static', variables.wallColor)
end

function createMouseJoint(world, x, y)
  for i, body in ipairs(world:getBodyList()) do
    if body:getFixtureList()[1]:testPoint(x, y) then
      mouseJoint = love.physics.newMouseJoint(body, x, y)
      return
    end
  end
end

function moveMouseJoint(x, y)
  if mouseJoint then
    mouseJoint:setTarget(x, y)
  end
end

function removeMouseJoint()
  if mouseJoint then
    mouseJoint:destroy()
    mouseJoint = nil
  end
end

function setBoxSizes(width, height)
  for i, box in ipairs(boxes) do
    box.fixture:destroy()
    box.shape = love.physics.newRectangleShape(width, height)
    box.fixture = love.physics.newFixture(box.body, box.shape)
  end
end

function setBoxBounciness(bounciness)
  for i, box in ipairs(boxes) do
    box.fixture:setRestitution(bounciness)
  end
end

function drawWorld(world)
  love.graphics.setLineWidth(8)
  for i, body in ipairs(world:getBodyList()) do
    local info = body:getUserData()
    local r, g, b = unpack(info.color)
    local points = { body:getWorldPoints(info.shape:getPoints()) }

    love.graphics.setColor(r, g, b)
    love.graphics.polygon('fill', points)

    love.graphics.setColor(r * 0.9, g * 0.9, b * 0.9)
    love.graphics.polygon('line', points)
  end
end
