local debugText = [[
%s

Click and drag the values to tweak them.
Click and drag the boxes to toss them around!
Press ` to toggle UI

%d FPS
%s
]]

function love.load()
  io.stdout:setvbuf('no')

  lovelive = require 'lovelive'

  require 'game'

  variables = {
    boxWidth = 80,
    boxHeight = 80,

    gravityX = 0,
    gravityY = 1000,
    bounciness = 0.5,

    boxColor = { 220, 100, 100 },
    wallColor = { 100, 220, 100 },
  }

  lovelive.init()
    .useMouse()
    .useToggleKey('`')
    .visible(true)

  world = love.physics.newWorld(0, 1000)
  populateWorld(world)
end

function love.mousepressed(x, y, button)
  if button == 1 then
    createMouseJoint(world, x, y)
  end
end

function love.mousemoved(x, y, dx, dy)
  if love.mouse.isDown(1) and lovelive.getInputState() ~= 'busy' then
    moveMouseJoint(x, y)
  end
end

function love.mousereleased(x, y)
  removeMouseJoint()
end

function love.update(dt)
  lovelive
    .group('Box')
      .number('Width', variables, 'boxWidth', 1, 1)
        .afterChanged(function(width) setBoxSizes(width, variables.boxHeight) end)
      .number('Height', variables, 'boxHeight', 1, 1)
        .afterChanged(function(height) setBoxSizes(variables.boxWidth, height) end)
      .number('Bounciness', variables, 'bounciness', 0.001, 0)
        .whileChanged(function(bounciness) setBoxBounciness(bounciness) end)

      .action('Size to Square', function()
        local size = math.floor((variables.boxWidth + variables.boxHeight) / 2)
        variables.boxWidth, variables.boxHeight = size, size
        setBoxSizes(size, size)
      end)

    .group('World')
      .number('Gravity X', variables, 'gravityX', 10)
        .whileChanged(function() world:setGravity(variables.gravityX, variables.gravityY) end)
      .number('Gravity Y', variables, 'gravityY', 10)
        .whileChanged(function() world:setGravity(variables.gravityX, variables.gravityY) end)

    .group('Colors')
      .object('Box Color', variables.boxColor, 1, 0, 255)
      .object('Wall Color', variables.wallColor, 1, 0, 255)

    .update(dt)

  world:update(dt)
end

function love.draw()
  love.graphics.setBackgroundColor(255, 255, 255)

  drawWorld(world)

  lovelive.draw()

  local fps = love.timer.getFPS()
  local screenWidth = love.graphics.getWidth()
  love.graphics.setColor(50, 50, 50)
  love.graphics.setNewFont(24)
  love.graphics.printf(debugText:format(lovelive.version, fps, lovelive.getInputState()), -10, 10, screenWidth, 'right')
end
